
#include "combinatorialgame.h"
using namespace std;

namespace AlgorithMan
{


    CombinatorialGamePosition::CombinatorialGamePosition()
    {
    }

    CombinatorialGamePosition::~CombinatorialGamePosition()
    {
    }

    bool CombinatorialGamePosition::WinningPosition()
    {
        return PlayerWon() != 0;
    }

    ostream& operator<<(ostream& os, const CombinatorialGamePosition& p)
    {
        p.WriteToStream(os);
    }



    CombinatorialGame::CombinatorialGame()
    {
    }

    CombinatorialGame::~CombinatorialGame()
    {
    }



    CombinatorialGameAI::CombinatorialGameAI()
    {
    }

    CombinatorialGameAI::~CombinatorialGameAI()
    {
    }

    // note that this would result in an infinite recursion on games which are not well-founded
    int CombinatorialGameAI::RatePosition(CombinatorialGamePosition* fromposition, int player)
    {
        if(fromposition->WinningPosition())
            return fromposition->PlayerWon() == player ? 1 : -1;


        // which move would the player, whose turn it is now, choose?
        set<CombinatorialGamePosition*> moves = fromposition->PossibleMoves();
        if(moves.size() <= 0)
            return 0; // not a winning position (that case is above), but no move possible => draw


        int positionplayer = fromposition->PlayersTurn();
        int bestrating = -1;
        for(set<CombinatorialGamePosition*>::iterator m = moves.begin(); m != moves.end(); m++)
        {
            int mrating = RatePosition(*m, positionplayer);
            if(mrating > bestrating)
            {
                bestrating = mrating;
                if(mrating == 1)         // alpha-beta cut
                    break;
            }
        }
        for(set<CombinatorialGamePosition*>::iterator m = moves.begin(); m != moves.end(); m++)
            delete(*m);
        moves.clear();


        return player == positionplayer ? bestrating : -1*bestrating; // neg-max
    }

    CombinatorialGamePosition* CombinatorialGameAI::ChooseMove(CombinatorialGamePosition* fromposition)
    {
        int player = fromposition->PlayersTurn();
        set<CombinatorialGamePosition*> moves = fromposition->PossibleMoves();
        if(moves.size() <= 0)
            return NULL;

        CombinatorialGamePosition* chosenmove = *moves.begin();
        int bestrating = -1;

        for(set<CombinatorialGamePosition*>::iterator m = moves.begin(); m != moves.end(); m++)
        {
            int mrating = RatePosition(*m, player);
            if(mrating > bestrating)
            {
                chosenmove = *m;
                bestrating = mrating;
                if(mrating == 1)
                    break;
            }
        }

        moves.erase(chosenmove);
        for(set<CombinatorialGamePosition*>::iterator m = moves.begin(); m != moves.end(); m++)
            delete(*m);
        moves.clear();

        return chosenmove;
    }


}